#!/bin/bash

set -e

encode_file() {
  local file="$1"
  if [ ! -f "$file" ]; then
    return 1
  fi
  base64 -i "$file"
}

encode_directory() {
  local dir="$1"
  local result="{}"
  if [ -d "$dir" ]; then
    result="{"
    for file in "$dir"/*.yaml; do
      [ -f "$file" ] || continue
      filename=$(basename "$file")
      encoded_file=$(encode_file "$file") || continue
      result+="\"$filename\": \"$encoded_file\","  
    done
    result="${result%,}" 
    result+="}"
    echo "$result"
    return 0
  fi
  return 1
}

if [ $# -lt 1 ]; then
  echo "Usage: $0 <DIRECTORY_PATH> <LUNAR_API_KEY: optional> <LUNAR_GATEWAY_IP: optional (default: hosted-gateway-control.lunar.dev)>"
  exit 1
fi

DIRECTORY_PATH="$1"
API_KEY="$2"
GATEWAY_IP_URL="${3:-hosted-gateway-control.lunar.dev}"

APPLY_FLOWS_URL="https://$GATEWAY_IP_URL/apply_flows"
X_LUNAR_API_KEY="x-lunar-api-key"

echo "Directory path: $DIRECTORY_PATH"
echo "Apply Flows URL: $APPLY_FLOWS_URL"
echo "API Key: $API_KEY"

if [ ! -d "$DIRECTORY_PATH" ]; then
  echo "Error: Directory does not exist"
  exit 1
fi

JSON_PAYLOAD="{"

add_json_field() {
  local key="$1"
  local value="$2"
  if [ -n "$value" ]; then
    JSON_PAYLOAD+="\"$key\": $value,"
  fi
}

metrics=$(encode_file "$DIRECTORY_PATH/metrics.yaml") && add_json_field "metrics" "\"$metrics\""
gateway_config=$(encode_file "$DIRECTORY_PATH/gateway_config.yaml") && add_json_field "gateway_config" "\"$gateway_config\""
flows=$(encode_directory "$DIRECTORY_PATH/flows") && add_json_field "flows" "$flows"
quotas=$(encode_directory "$DIRECTORY_PATH/quotas") && add_json_field "quotas" "$quotas"
path_params=$(encode_directory "$DIRECTORY_PATH/path_params") && add_json_field "path_params" "$path_params"

JSON_PAYLOAD="${JSON_PAYLOAD%,}}"

if [ -z "$API_KEY" ]; then
  echo "No API key provided. Here is the JSON payload:"
  echo "$JSON_PAYLOAD"
  exit 0
fi

type curl >/dev/null 2>&1 && HTTP_CLIENT="curl"
type wget >/dev/null 2>&1 && HTTP_CLIENT="wget"

if [ -z "$HTTP_CLIENT" ]; then
  echo "No HTTP client (curl or wget) found. Here is the JSON payload:"
  echo "$JSON_PAYLOAD"
  exit 0
fi

if [ "$HTTP_CLIENT" = "curl" ]; then
  echo "[[Using curl to apply flows]]"
  echo ""
  
  response_file=$(mktemp)
  http_code=$(curl -X PUT "$APPLY_FLOWS_URL" \
    -H "Content-Type: application/json" \
    -H "$X_LUNAR_API_KEY: $API_KEY" \
    -d "$JSON_PAYLOAD" \
    -w "%{http_code}" -o "$response_file" -s)

  response_body=$(cat "$response_file")
  rm -f "$response_file"

  if [[ "$http_code" -ne 200 ]]; then
    echo "Error: Failed to apply flows. HTTP code: $http_code"
    if command -v jq >/dev/null 2>&1; then
      echo "Response Body (Formatted by jq):"
      echo "$response_body" | jq .
    else
      echo "Response Body:"
      echo "$response_body"
    fi
    exit 1
  fi
  echo "✅ Flows applied successfully"

elif [ "$HTTP_CLIENT" = "wget" ]; then
  echo "[[Using wget to apply flows]]"
  echo ""

  response_file=$(mktemp)
  wget --server-response --content-on-error --header="Content-Type: application/json" \
       --header="$X_LUNAR_API_KEY: $API_KEY" \
       --method=PUT \
       --body-data="$JSON_PAYLOAD" \
       -O "$response_file" "$APPLY_FLOWS_URL" 2> >(tee /tmp/wget_headers.log >&2 || true) || true

  # Extract HTTP status code
  http_code=$(grep -o "HTTP/[0-9.]* [0-9]*" /tmp/wget_headers.log | tail -n1 | awk '{print $2}')
  response_body=$(cat "$response_file")
  rm -f "$response_file" /tmp/wget_headers.log

  if [[ -z "$http_code" || "$http_code" -ne 200 ]]; then
    echo "Error: Failed to apply flows. HTTP code: $http_code"
    if command -v jq >/dev/null 2>&1; then
      echo "Response Body (Formatted):"
      echo "$response_body" | jq .
    else
      echo "Response Body:"
      echo "$response_body"
    fi
    exit 1
  fi
  echo "✅ Flows applied successfully"
fi